@@ -52,6 +52,20 @@
 
 /**@{*/
 
+/* Define column names */
+#ifdef OPR2
+#define COLNAME_OBJ_RA "TRG_RA"
+#define COLNAME_OBJ_DEC "TRG_DEC"
+#else
+#define COLNAME_OBJ_RA "OBJ_RA"
+#define COLNAME_OBJ_DEC "OBJ_DEC"
+#endif
+
+static cpl_error_code qmost_fibtab_metqc (
+    cpl_table *tab,
+    cpl_propertylist *hdr,
+    double mjd);
+
 /*----------------------------------------------------------------------------*/
 /**
  * @brief   Load the FIBINFO table from an input frame.
@@ -98,32 +112,34 @@
  *                                    columns had an incorrect data
  *                                    type.
  *
- * @par Input FITS Header Information:
- *   - <b>ESO INS SHUTn STAT</b>
- *
  * @par Input FIBINFO table columns:
  *   - <b>FIB_ID</b>
  *   - <b>FIB_ROOT</b>
  *   - <b>SLIT_POS</b>
  *
+ * @par Output QC Parameters:
+ *   - <b>ONSKYDIST MED</b> (arcsec): Median fibre positioning
+ *     error in observation.
+ *   - <b>ONSKYDIST OUTPERC</b> (percent): Fraction of fibres with
+ *     positioning error greater than 0.5 arcsec.
+ *   - <b>METPREC MED</b> (arcsec): Median metrology system
+ *     precision in observation.
+ *
  * @par Output FIBINFO table columns:
  *   The following columns are populated for the simultaneous
  *   calibration fibres.
  *   - <b>FIB_ID</b>: The unique fibre ID of the object.  Populated
  *     with the otherwise invalid flag value of zero for the
  *     simultaneous calibration fibres.
- *   - <b>FIB_ST</b>: Fibre status.  Populated with a value equal to
- *     the simultaneous calibration fibre shutter status (ESO INS
- *     SHUTn STAT), i.e. 1 meaning parked but can see light if the
- *     shutter is open, and 0 meaning disabled and cannot see light if
- *     the shutter is closed.
+ *   - <b>FIB_ST</b>: Fibre status.  Populated with a value of 1,
+ *     meaning parked but can see light.
  *   - <b>FIB_USE</b>: Fibre use.  Populated with a special value of
  *     zero for simultaneous calibration fibre.
  *   - <b>FIB_ROOT</b>: Specifies where the fibre is routed.
  *     Populated with the spectrograph ID from the parameter shuffle.
  *   - <b>SLIT_POS</b>: Slit position of the fibre.  The simultaneous
- *     calibration files are given their correct slit positions 1-5
- *     and 818-822.
+ *     calibration files are given their correct slit positions 1-6
+ *     and 819-824.
  *
  * @note    The shuffle algorithm also deletes entries for which there
  *          is no fibre.
@@ -140,6 +156,7 @@ cpl_error_code qmost_fibtabload (
     cpl_propertylist **hdr)
 {
     const char *filename;
+    cpl_propertylist *prihdr = NULL;
     cpl_table *intab = NULL;
     cpl_propertylist *inhdr = NULL;
     cpl_table *mettab = NULL;
@@ -162,14 +179,18 @@ cpl_error_code qmost_fibtabload (
     const char *colname, *colunit;
     cpl_type coltype;
 
-    cpl_propertylist *prihdr = NULL;
-    char *key = NULL;
-    cpl_errorstate prestate;
-    int shut_open, min_pos, nsimucal = 0, isnull, fib_st;
+    int min_pos, nsimucal = 0, isnull;
     int slit_pos;
 
+    cpl_errorstate prestate;
+    double mjd;
+
 #undef TIDY
 #define TIDY                                    \
+    if(prihdr != NULL) {                        \
+        cpl_propertylist_delete(prihdr);        \
+        prihdr = NULL;                          \
+    }                                           \
     if(intab != NULL) {                         \
         cpl_table_delete(intab);                \
         intab = NULL;                           \
@@ -201,14 +222,6 @@ cpl_error_code qmost_fibtabload (
     if(metnames != NULL) {                      \
         cpl_array_delete(metnames);             \
         metnames = NULL;                        \
-    }                                           \
-    if(prihdr != NULL) {                        \
-        cpl_propertylist_delete(prihdr);        \
-        prihdr = NULL;                          \
-    }                                           \
-    if(key != NULL) {                           \
-        cpl_free(key);                          \
-        key = NULL;                             \
     }
 
     /* Check required inputs and outputs */
@@ -224,6 +237,15 @@ cpl_error_code qmost_fibtabload (
                                      "couldn't get filename from frame");
     }
 
+    /* Primary header */
+    prihdr = cpl_propertylist_load(filename, 0);
+    if(prihdr == NULL) {
+        TIDY;
+        return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                     "couldn't read file %s primary HDU",
+                                     filename);
+    }
+    
     /* Get extension number */
     inext = cpl_fits_find_extension(filename,
                                     QMOST_FIBINFO_EXTNAME);
@@ -618,53 +640,6 @@ cpl_error_code qmost_fibtabload (
         }
 
         if(nsimucal > 0) {
-            /* Figure out if simucal fibres are illuminated using PHDU
-             * keywords.  We first check if any of the lamps are on, and
-             * then if the appropriate shutter is open. */
-            prihdr = cpl_propertylist_load(filename, 0);
-            if(prihdr == NULL) {
-                TIDY;
-                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
-                                             "couldn't load primary FITS "
-                                             "header for %s",
-                                             filename);
-            }
-
-            key = cpl_sprintf("ESO INS SHUT%d STAT", shuffle);
-            if(key == NULL) {
-                TIDY;
-                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
-                                             "couldn't format shutter FITS "
-                                             "header key for %d",
-                                             shuffle);
-            }
-
-            prestate = cpl_errorstate_get();
-
-            if(qmost_cpl_propertylist_get_int(prihdr,
-                                              key,
-                                              &shut_open) != CPL_ERROR_NONE) {
-                cpl_msg_warning(cpl_func,
-                                "couldn't read simucal shutter status "
-                                "%s[0]:%s (%s), assuming open",
-                                filename,
-                                key,
-                                cpl_error_get_message());
-
-                cpl_errorstate_set(prestate);
-
-                fib_st = 1;
-            }
-            else {
-                fib_st = shut_open ? 1 : 0;
-            }
-
-            cpl_free(key);
-            key = NULL;
-
-            cpl_propertylist_delete(prihdr);
-            prihdr = NULL;
-
             /* Insert blanks for the simucal fibres at start, set FIB_USE=0,
                and FIB_ROOT and SLIT_POS to appropriate values. */
             if(qmost_cpl_table_insert_blank_window(
@@ -693,7 +668,7 @@ cpl_error_code qmost_fibtabload (
                                                 "FIB_ST",
                                                 0,
                                                 nsimucal,
-                                                fib_st) != CPL_ERROR_NONE) {
+                                                1) != CPL_ERROR_NONE) {
                 TIDY;
                 return cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "failed to set FIB_ST for "
@@ -766,7 +741,7 @@ cpl_error_code qmost_fibtabload (
                                                 "FIB_ST",
                                                 nrowstmp,
                                                 nsimucal,
-                                                fib_st) != CPL_ERROR_NONE) {
+                                                1) != CPL_ERROR_NONE) {
                 TIDY;
                 return cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "failed to set FIB_ST for "
@@ -809,6 +784,20 @@ cpl_error_code qmost_fibtabload (
             }
         }
 
+        /* Metrology QC */
+        prestate = cpl_errorstate_get();
+        
+        if(qmost_pfits_get_mjd_obs(prihdr, &mjd) != CPL_ERROR_NONE) {
+            cpl_errorstate_set(prestate);
+            mjd = -1.0;
+        }
+        
+         if(qmost_fibtab_metqc(shuftab, inhdr, mjd) != CPL_ERROR_NONE) {
+            TIDY;
+            return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                         "METROLOGY QC failed");
+        }
+
         *tab = shuftab;
 
         cpl_table_delete(intab);
@@ -823,6 +812,9 @@ cpl_error_code qmost_fibtabload (
         mettab = NULL;
     }
 
+    cpl_propertylist_delete(prihdr);
+    prihdr = NULL;
+    
     *hdr = inhdr;
 
     return CPL_ERROR_NONE;
@@ -1363,15 +1355,6 @@ cpl_error_code qmost_fibtab_newcols (
         }
     }
 
-    /* Define column names */
-#ifdef OPR2
-#define COLNAME_OBJ_RA "TRG_RA"
-#define COLNAME_OBJ_DEC "TRG_DEC"
-#else
-#define COLNAME_OBJ_RA "OBJ_RA"
-#define COLNAME_OBJ_DEC "OBJ_DEC"
-#endif
-
     /* Loop over objects */
     for(irow = 0; irow < nrows; irow++) {
         /* Set EXPTIME */
@@ -1436,7 +1419,7 @@ cpl_error_code qmost_fibtab_newcols (
             TIDY;
             return cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                          "could not read %s for row %d",
-                                         COLNAME_OBJ_RA, irow+1);
+                                         COLNAME_OBJ_DEC, irow+1);
         }
 
         /* Check coordinates */
@@ -1848,4 +1831,514 @@ cpl_error_code qmost_fibtab_arcqc (
     return CPL_ERROR_NONE;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief   Populate METROLOGY QC parameters.
+ *
+ * Computes METROLOGY QC parameters.  This doesn't really have
+ * anything to do with data reduction, it's purely based on quantities
+ * already given in the raw files, but this is apparently the only
+ * place it can be implemented to make the parameters accessible to
+ * the QC system.
+ *
+ * @param   tab        (Given)    The FIBINFO table.
+ * @param   hdr        (Given)    FITS header into which QC parameters
+ *                                should be emitted.
+ * @param   mjd        (Given)    MJD of observation for proper motion
+ *                                correction, or < 0 if unknown, in
+ *                                which case no proper motion
+ *                                correction will be done.
+ *
+ * @return  cpl_error_code
+ *
+ * @retval  CPL_ERROR_NONE            If everything is OK.
+ * @retval  CPL_ERROR_DATA_NOT_FOUND  If no FIBINFO extension was
+ *                                    found in the input file, or one
+ *                                    of the required columns didn't
+ *                                    exist.
+ * @retval  CPL_ERROR_NULL_INPUT      If one of the required input or
+ *                                    output pointers was NULL.
+ * @retval  CPL_ERROR_TYPE_MISMATCH   If one of the FIBINFO table
+ *                                    columns had an incorrect data
+ *                                    type.
+ *
+ * @par Input FIBINFO table columns:
+ *   - <b>FIB_ID</b>
+ *   - <b>FIB_ROOT</b>
+ *   - <b>OBJ_RA</b>
+ *   - <b>OBJ_DEC</b>
+ *   - <b>OBJ_PMRA</b>
+ *   - <b>OBJ_PMDE</b>
+ *   - <b>OBJ_EPOC</b>
+ *   - <b>OBJ_ROFF</b>
+ *   - <b>OBJ_DOFF</b>
+ *   - <b>OBS_RA</b>
+ *   - <b>OBS_DEC</b>
+ *   - <b>FIB_SKY_ACC</b>
+ *
+ * @par Output QC Parameters:
+ *   - <b>ONSKYDIST MED</b> (arcsec): Median fibre positioning
+ *     error in observation.
+ *   - <b>ONSKYDIST OUTPERC</b> (percent): Fraction of fibres with
+ *     positioning error greater than 0.5 arcsec.
+ *   - <b>METPREC MED</b> (arcsec): Median metrology system
+ *     precision in observation.
+ *
+ * @author  Jonathan Irwin, CASU
+ */
+/*----------------------------------------------------------------------------*/
+
+static cpl_error_code qmost_fibtab_metqc (
+    cpl_table *tab,
+    cpl_propertylist *hdr,
+    double mjd)
+{
+    int do_pos = 0;
+    int do_pmcorr = 0;
+    int do_off = 0;
+    int do_acc = 0;
+    double obs_epoc;
+    
+    int irow, nrows;
+
+    double *poserr = NULL;
+    double *skyacc = NULL;
+    int nposerr, nskyacc;
+
+    int fib_st, fib_use, isnull;
+
+    double obj_ra, obj_dec, obs_ra, obs_dec;
+    int obj_ra_isnull, obj_dec_isnull, obs_ra_isnull, obs_dec_isnull;
+
+    double obj_pmra, obj_pmde, obj_epoc;
+    int obj_pmra_isnull, obj_pmde_isnull, obj_epoc_isnull;
+
+    double obj_roff, obj_doff;
+    int obj_roff_isnull, obj_doff_isnull;
+    
+    double sa, ca, sd, cd, dep;
+    double vobj[3], dvdt[3], vobs[3], vx[3];
+    int i;
+    double theta;
+
+    double fib_sky_acc;
+    int fib_sky_acc_isnull;
+
+    int nbad;
+    double med_poserr;
+    double med_skyacc;
+
+    cpl_ensure_code(tab != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(hdr != NULL, CPL_ERROR_NULL_INPUT);
+
+#undef TIDY
+#define TIDY                                    \
+    if(poserr != NULL) {                        \
+        cpl_free(poserr);                       \
+        poserr = NULL;                          \
+    }                                           \
+    if(skyacc != NULL) {                        \
+        cpl_free(skyacc);                       \
+        skyacc = NULL;                          \
+    }
+
+    /* Check columns exist.  If not, it's not an error. */
+    if(cpl_table_has_column(tab, COLNAME_OBJ_RA) &&
+       cpl_table_has_column(tab, COLNAME_OBJ_DEC) &&
+       cpl_table_has_column(tab, "OBS_RA") &&
+       cpl_table_has_column(tab, "OBS_DEC")) {
+        do_pos = 1;
+    }
+
+    if(mjd >= 0 &&
+       cpl_table_has_column(tab, "OBJ_PMRA") &&
+       cpl_table_has_column(tab, "OBJ_PMDE") &&
+       cpl_table_has_column(tab, "OBJ_EPOC")) {
+        do_pmcorr = 1;
+        obs_epoc = 2000.0 + (mjd - 51544.5) / 365.25;
+    }
+    else {
+        obs_epoc = 0.0;
+    }
+
+    if(cpl_table_has_column(tab, "OBJ_ROFF") &&
+       cpl_table_has_column(tab, "OBJ_DOFF")) {
+        do_off = 1;
+    }
+    
+    if(cpl_table_has_column(tab, "FIB_SKY_ACC")) {
+        do_acc = 1;
+    }
+
+    /* Get out now if there are no columns */
+    if(!do_pos && !do_acc) {
+        return CPL_ERROR_NONE;
+    }
+
+    /* Number of rows to process */
+    nrows = cpl_table_get_nrow(tab);
+
+    /* Allocate workspace */
+    poserr = cpl_malloc(nrows * sizeof(double));
+    skyacc = cpl_malloc(nrows * sizeof(double));
+
+    nposerr = 0;
+    nskyacc = 0;
+
+    for(irow = 0; irow < nrows; irow++) {
+        /* Should fibre be included? */
+        fib_st = cpl_table_get_int(tab, "FIB_ST", irow, &isnull);
+        if(isnull < 0) {
+            TIDY;
+            return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                         "failed to read %s column "
+                                         "for row %d",
+                                         "FIB_ST",
+                                         irow + 1);
+        }
+        else if(isnull > 0) {
+            /* Skip nulls */
+            continue;
+        }
+        else if(fib_st != 2) {
+            /* Skip any fibres that are not live */
+            continue;
+        }
+
+        fib_use = cpl_table_get_int(tab, "FIB_USE", irow, &isnull);
+        if(isnull < 0) {
+            TIDY;
+            return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                         "failed to read %s column "
+                                         "for row %d",
+                                         "FIB_USE",
+                                         irow + 1);
+        }
+        else if(isnull > 0) {
+            /* Skip nulls */
+            continue;
+        }
+        else if(fib_use == 0) {
+            /* Skip simucals */
+            continue;
+        }
+
+        if(do_pos) {
+            /* Get coordinates */
+            obj_ra = cpl_table_get(tab,
+                                   COLNAME_OBJ_RA,
+                                   irow,
+                                   &obj_ra_isnull);
+            if(obj_ra_isnull < 0) {
+                TIDY;
+                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                             "could not read %s for row %d",
+                                             COLNAME_OBJ_RA, irow+1);
+            }
+
+            obj_dec = cpl_table_get(tab,
+                                    COLNAME_OBJ_DEC,
+                                    irow,
+                                    &obj_dec_isnull);
+            if(obj_dec_isnull < 0) {
+                TIDY;
+                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                             "could not read %s for row %d",
+                                             COLNAME_OBJ_DEC, irow+1);
+            }
+
+            if(do_pmcorr) {
+                obj_pmra = cpl_table_get(tab,
+                                         "OBJ_PMRA",
+                                         irow,
+                                         &obj_pmra_isnull);
+                if(obj_pmra_isnull < 0) {
+                    TIDY;
+                    return cpl_error_set_message(cpl_func,
+                                                 cpl_error_get_code(),
+                                                 "could not read %s "
+                                                 "for row %d",
+                                                 "OBJ_PMRA", irow+1);
+                }
+
+                obj_pmde = cpl_table_get(tab,
+                                         "OBJ_PMDE",
+                                         irow,
+                                         &obj_pmde_isnull);
+                if(obj_pmde_isnull < 0) {
+                    TIDY;
+                    return cpl_error_set_message(cpl_func,
+                                                 cpl_error_get_code(),
+                                                 "could not read %s "
+                                                 "for row %d",
+                                                 "OBJ_PMDE", irow+1);
+                }
+
+                obj_epoc = cpl_table_get(tab,
+                                         "OBJ_EPOC",
+                                         irow,
+                                         &obj_epoc_isnull);
+                if(obj_epoc_isnull < 0) {
+                    TIDY;
+                    return cpl_error_set_message(cpl_func,
+                                                 cpl_error_get_code(),
+                                                 "could not read %s "
+                                                 "for row %d",
+                                                 "OBJ_EPOC", irow+1);
+                }
+            }
+            else {
+                obj_pmra = 0.0;
+                obj_pmde = 0.0;
+                obj_epoc = 0.0;
+                obj_pmra_isnull = 1;
+                obj_pmde_isnull = 1;
+                obj_epoc_isnull = 1;
+            }
+
+            if(do_off) {
+                obj_roff = cpl_table_get(tab,
+                                         "OBJ_ROFF",
+                                         irow,
+                                         &obj_roff_isnull);
+                if(obj_roff_isnull < 0) {
+                    TIDY;
+                    return cpl_error_set_message(cpl_func,
+                                                 cpl_error_get_code(),
+                                                 "could not read %s "
+                                                 "for row %d",
+                                                 "OBJ_ROFF", irow+1);
+                }
+
+                obj_doff = cpl_table_get(tab,
+                                         "OBJ_DOFF",
+                                         irow,
+                                         &obj_doff_isnull);
+                if(obj_doff_isnull < 0) {
+                    TIDY;
+                    return cpl_error_set_message(cpl_func,
+                                                 cpl_error_get_code(),
+                                                 "could not read %s "
+                                                 "for row %d",
+                                                 "OBJ_DOFF", irow+1);
+                }
+            }
+            else {
+                obj_roff = 0.0;
+                obj_doff = 0.0;
+                obj_roff_isnull = 1;
+                obj_doff_isnull = 1;
+            }
+            
+            obs_ra = cpl_table_get(tab,
+                                   "OBS_RA",
+                                   irow,
+                                   &obs_ra_isnull);
+            if(obs_ra_isnull < 0) {
+                TIDY;
+                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                             "could not read %s for row %d",
+                                             "OBS_RA", irow+1);
+            }
+
+            obs_dec = cpl_table_get(tab,
+                                    "OBS_DEC",
+                                    irow,
+                                    &obs_dec_isnull);
+            if(obs_dec_isnull < 0) {
+                TIDY;
+                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                             "could not read %s for row %d",
+                                             "OBS_DEC", irow+1);
+            }
+
+            /* Check coordinates */
+            if(obj_ra_isnull == 0 && obj_dec_isnull == 0 &&
+               obs_ra_isnull == 0 && obs_dec_isnull == 0) {
+                /* Unit vector obj */
+                sa = sin(obj_ra * CPL_MATH_RAD_DEG);
+                ca = cos(obj_ra * CPL_MATH_RAD_DEG);
+                sd = sin(obj_dec * CPL_MATH_RAD_DEG);
+                cd = cos(obj_dec * CPL_MATH_RAD_DEG);
+
+                vobj[0] = ca * cd;
+                vobj[1] = sa * cd;
+                vobj[2] = sd;
+
+                /* Proper motion correction, if available */
+                if(do_pmcorr) {
+                    /* Replace any missing input data */
+                    if(obj_pmra_isnull > 0 || !isfinite(obj_pmra)) {
+                        obj_pmra = 0.0;
+                    }
+                    
+                    if(obj_pmde_isnull > 0 || !isfinite(obj_pmde)) {
+                        obj_pmde = 0.0;
+                    }
+
+                    if(obj_epoc_isnull > 0 || !isfinite(obj_epoc)) {
+                        obj_epoc = 2000.0;
+                    }
+
+                    /* Velocity vector, mas/yr */
+                    dvdt[0] = -obj_pmra * sa - obj_pmde * ca*sd;
+                    dvdt[1] =  obj_pmra * ca - obj_pmde * sa*sd;
+                    dvdt[2] =  obj_pmde * cd;
+
+                    /* Epoch difference */
+                    dep = obs_epoc - obj_epoc;
+
+                    /* Convert units */
+                    dep *= CPL_MATH_RAD_DEG / (1000 * 3600);
+                    
+                    /* Apply proper motion */
+                    for(i = 0; i < 3; i++) {
+                        vobj[i] += dep * dvdt[i];
+                    }
+                }
+
+                /* Offsets are applied to observed position.   Doesn't
+                 * really matter which we do, and it made for a
+                 * cleaner calculation of source position above. */
+                if(do_off) {
+                    /* Replace any missing input data */
+                    if(obj_roff_isnull > 0 || !isfinite(obj_roff)) {
+                        obj_roff = 0.0;
+                    }
+
+                    if(obj_doff_isnull > 0 || !isfinite(obj_doff)) {
+                        obj_doff = 0.0;
+                    }
+
+                    /* The offset here seems to be "time", i.e. these
+                     * specify a raw displacement in the angles, based
+                     * on my inspection of the files.  This is not
+                     * correct if it specifies a displacement on sky,
+                     * where the RA offsets would be too small at
+                     * large abs(dec).  It seems like a mistake to me
+                     * but we need to reproduce the mistake here when
+                     * comparing to metrology output. */
+                    obs_ra -= obj_roff / 3600.0;
+                    obs_dec -= obj_doff / 3600.0;
+                }
+                
+                /* Unit vector obs */
+                sa = sin(obs_ra * CPL_MATH_RAD_DEG);
+                ca = cos(obs_ra * CPL_MATH_RAD_DEG);
+                sd = sin(obs_dec * CPL_MATH_RAD_DEG);
+                cd = cos(obs_dec * CPL_MATH_RAD_DEG);
+
+                vobs[0] = ca * cd;
+                vobs[1] = sa * cd;
+                vobs[2] = sd;
+
+                /* Form vector product vobj x vobs */
+                vx[0] = vobj[1] * vobs[2] - vobj[2] * vobs[1];
+                vx[1] = vobj[2] * vobs[0] - vobj[0] * vobs[2];
+                vx[2] = vobj[0] * vobs[1] - vobj[1] * vobs[0];
+
+                /* Angular separation theta is given by
+                 * |u x v| = |u||v| sin theta
+                 *  u . v  = |u||v| cos theta
+                 * so theta = atan(|u x v| / u . v) */
+                theta = atan2(sqrt(vx[0] * vx[0] +
+                                   vx[1] * vx[1] +
+                                   vx[2] * vx[2]),
+                              vobj[0] * vobs[0] +
+                              vobj[1] * vobs[1] +
+                              vobj[2] * vobs[2]);
+
+                if(isfinite(theta)) {
+                    /* Wrap angle into [0, 2*pi] */
+                    if(theta < 0) {
+                        theta += 2.0 * CPL_MATH_PI;
+                    }
+
+                    poserr[nposerr] = theta * CPL_MATH_DEG_RAD * 3600.0;
+                    nposerr++;
+                }
+            }
+        }
+
+        if(do_acc) {
+            /* Get FIB_SKY_ACC */
+            fib_sky_acc = cpl_table_get(tab,
+                                        "FIB_SKY_ACC",
+                                        irow,
+                                        &fib_sky_acc_isnull);
+            if(fib_sky_acc_isnull < 0) {
+                TIDY;
+                return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                             "could not read %s for row %d",
+                                             "FIB_SKY_ACC", irow+1);
+            }
+
+            if(fib_sky_acc_isnull == 0 && isfinite(fib_sky_acc)) {
+                skyacc[nskyacc] = fib_sky_acc;
+                nskyacc++;
+            }
+        }
+    }
+
+    if(nposerr > 0) {
+        /* Fraction greater than specified 0.5 arcsec error */
+        nbad = 0;
+
+        for(irow = 0; irow < nposerr; irow++) {
+            if(poserr[irow] > 0.5) {
+                nbad++;
+            }
+        }
+
+        /* Median positioning error */
+        if(qmost_dmed(poserr, NULL, nposerr, &med_poserr) != CPL_ERROR_NONE) {
+            TIDY;
+            return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                         "could not compute median "
+                                         "position error");
+        }
+
+        cpl_propertylist_update_double(hdr,
+                                       "ESO QC ONSKYDIST MED",
+                                       med_poserr);
+        cpl_propertylist_set_comment(hdr,
+                                     "ESO QC ONSKYDIST MED",
+                                     "[arcsec] Median fibre "
+                                     "positioning error");
+
+        cpl_propertylist_update_double(hdr,
+                                       "ESO QC ONSKYDIST OUTPERC",
+                                       100.0 * nbad / ((double) nposerr));
+        cpl_propertylist_set_comment(hdr,
+                                     "ESO QC ONSKYDIST OUTPERC",
+                                     "[percent] fibres outside 0.5 arcsec");
+    }
+
+    if(nskyacc > 0) {
+        /* Median metrology precision */
+        if(qmost_dmed(skyacc, NULL, nskyacc, &med_skyacc) != CPL_ERROR_NONE) {
+            TIDY;
+            return cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                         "could not compute median "
+                                         "metrology precision");
+        }
+
+        cpl_propertylist_update_double(hdr,
+                                       "ESO QC METPREC MED",
+                                       med_skyacc);
+        cpl_propertylist_set_comment(hdr,
+                                     "ESO QC METPREC MED",
+                                     "[arcsec] Median metrology "
+                                     "precision");
+    }
+
+    cpl_free(poserr);
+    poserr = NULL;
+
+    cpl_free(skyacc);
+    skyacc = NULL;
+
+    return CPL_ERROR_NONE;
+}
+
 /**@}*/
