@@ -490,7 +490,7 @@ cpl_error_code qmost_model_psf_full (
         qmost_filt1d(usefwhm, tr[ifib-1].npos, medfil, linfil, -1000);
 
         /* Information for neighbouring fibres */
-        if(ifib >= 2) {
+        if(ifib >= 2 && tr[ifib-2].live) {
             coefsa = tr[ifib-2].coefs;
             yrefa = tr[ifib-2].yref;
 
@@ -530,7 +530,7 @@ cpl_error_code qmost_model_psf_full (
             yrefa = 0;
         }
 
-        if(ifib < nh) {
+        if(ifib < nh && tr[ifib].live) {
             coefsb = tr[ifib].coefs;
             yrefb = tr[ifib].yref;
 
