@@ -467,10 +467,10 @@ cpl_error_code qmost_skysub_vshiftpca (
     }
 
 /* These macros are used to check the FIB_USE column */
-#define ISSKY(a) (a == 3)
-#define ISTARGET(a) (a == 1)
-#define USEFORSKY(a) ((a == 3 || a == 1))
-#define CORRECTSKY(a) ((a == 3 || a == 1 || a == 4 || a == 7 || a == 8))
+#define ISSKY(a) ((a) == 3)
+#define ISTARGET(a) ((a) == 1)
+#define USEFORSKY(a) ((a) == 3 || (a) == 1)
+#define CORRECTSKY(a) ((a) != 0)
 
     /* Get input 2D spectra */
     naxis[0] = cpl_image_get_size_x(spec_img);
