@@ -94,7 +94,7 @@ static void test_qmost_fibtabload(void)
     /* Not a FITS file */
     cpl_frame_set_filename(frame, tmpfile);
     code = qmost_fibtabload(frame, 0, &table, &hdr);
-    cpl_test_eq_error(code, CPL_ERROR_FILE_IO);
+    cpl_test_eq_error(code, CPL_ERROR_BAD_FILE_FORMAT);
 
     /* Create dummy PHDU so file is now valid FITS */
     outhdr = cpl_propertylist_new();
